{*****************************************************************************
*                                                                            *
*  FGFW4.PAS                                                                 *
*                                                                            *
*  Demonstrate manual font loading.                                          *
*                                                                            *
*****************************************************************************}

program FGFW4;

uses WinTypes, WinProcs, FGWin, FGFWin;

const
  AppName = 'FGFW4';
  FONTSIZE = 8258;

var
  cxClient, cyClient : word;
  dc   : hDC;
  hpal : hPalette;
  hvb  : integer;
  font_array : array [1..FONTSIZE] of byte;

{*****************************************************************************
*                                                                            *
*  WindowProc                                                                *
*                                                                            *
*  Window procedure to handle messages sent to the window.                   *
*                                                                            *
*****************************************************************************}

function WindowProc(window : hwnd; message : word;
                    wParam : word; lParam : longint): longint; export;
var
  ps : tPaintStruct;
  stream : file;

begin

  WindowProc := 0;

  case message of

    WM_CREATE:
    begin
      dc := GetDC(window);
      fg_setdc(dc);
      hpal := fg_defpal;
      fg_realize(hpal);

      fg_vbinit;
      hvb := fg_vballoc(640,480);
      fg_vbopen(hvb);
      fg_vbcolors;

      {$I-}
      assign(stream,'MODERN28.FGF');
      reset(stream,1);
      if IOResult <> 0 then
      begin
        MessageBox(window,'Unable to load font file.','Error',MB_ICONSTOP or MB_OK);
        DestroyWindow(window);
        Exit;
      end;
      blockread(stream,font_array,FONTSIZE);
      close(stream);
      {$I+}
      fgf_define(font_array);

      fg_setcolor(255);
      fg_fillpage;

      fg_setcolor(19);
      fg_move(fg_getmaxx div 2,fg_getmaxy div 2);
      fgf_justify(0,0);
      fgf_print('FG/Fonts',8);

      Exit;
    end;

    WM_PAINT:
    begin
      BeginPaint(window,ps);
      fg_vbscale(0,fg_getmaxx,0,fg_getmaxy,0,cxClient-1,0,cyClient-1);
      EndPaint(window,ps);
      Exit;
    end;

    WM_SETFOCUS:
    begin
      fg_realize(hpal);
      InvalidateRect(window,nil,TRUE);
      Exit;
    end;

    WM_SIZE:
    begin
      cxClient := LOWORD(lParam);
      cyClient := HIWORD(lParam);
      Exit;
    end;

    WM_DESTROY:
    begin
      fg_vbclose;
      fg_vbfree(hvb);
      fg_vbfin;
      DeleteObject(hpal);
      ReleaseDC(window,dc);
      PostQuitMessage(0);
      Exit;
    end;

  end;
  WindowProc := DefWindowProc(window,message,wParam,lParam);
end;

{****************************************************************************}

procedure WinMain;

var
  window      : hWnd;
  message     : tMsg;
  WindowClass : tWndClass;

begin
  if hPrevInst = 0 then
  begin
    WindowClass.style         := CS_HREDRAW OR CS_VREDRAW OR CS_OWNDC;
    WindowClass.lpfnWndProc   := @WindowProc;
    WindowClass.cbClsExtra    := 0;
    WindowClass.cbWndExtra    := 0;
    WindowClass.hInstance     := hInstance;
    WindowClass.hIcon         := LoadIcon(0,IDI_APPLICATION);
    WindowClass.hCursor       := LoadCursor(0,IDC_ARROW);
    WindowClass.hbrBackground := 0;
    WindowClass.lpszMenuName  := nil;
    WindowClass.lpszClassName := AppName;
    if not RegisterClass(WindowClass) then Halt(255);
  end;

  window := CreateWindow(
    AppName,                  { window class name }
    'Example FGFW4',          { window caption }
    WS_OVERLAPPEDWINDOW,      { window style }
    CW_USEDEFAULT,            { initial x position }
    CW_USEDEFAULT,            { initial y position }
    CW_USEDEFAULT,            { initial x size }
    CW_USEDEFAULT,            { initial y size }
    0,                        { parent window handle }
    0,                        { window menu handle }
    hInstance,                { program instance handle }
    nil);                     { creation parameters }

  ShowWindow(window,CmdShow);
  UpdateWindow(window);

  while GetMessage(message,0,0,0) do
  begin
    TranslateMessage(message);
    DispatchMessage(message);
  end;

  Halt(message.wParam);
end;

begin
  WinMain;
end.
