{*****************************************************************************
*                                                                            *
*  FGFW7.PAS                                                                 *
*                                                                            *
*  Demonstrate shadowed characters.                                          *
*                                                                            *
*****************************************************************************}

program FGFW7;

uses WinTypes, WinProcs, FGWin, FGFWin;

const
  AppName = 'FGFW7';

var
  cxClient, cyClient : word;
  dc   : hDC;
  hpal : hPalette;
  hvb  : integer;

{*****************************************************************************
*                                                                            *
*  WindowProc                                                                *
*                                                                            *
*  Window procedure to handle messages sent to the window.                   *
*                                                                            *
*****************************************************************************}

function WindowProc(window : hwnd; message : word;
                    wParam : word; lParam : longint): longint; export;
var
  ps : tPaintStruct;
  midx, midy : integer;
  message_text : string;

begin

  WindowProc := 0;

  case message of

    WM_CREATE:
    begin
      dc := GetDC(window);
      fg_setdc(dc);
      hpal := fg_defpal;
      fg_realize(hpal);

      fg_vbinit;
      hvb := fg_vballoc(640,480);
      fg_vbopen(hvb);
      fg_vbcolors;

      if fgf_load('AUSTIN36.FGF'+chr(0)) = 0 then
      begin
        MessageBox(window,'Unable to load font file.','Error',MB_ICONSTOP or MB_OK);
        DestroyWindow(window);
        Exit;
      end;

      fg_setcolor(255);
      fg_fillpage;

      midx := fg_getmaxx div 2;
      midy := fg_getmaxy div 2;
      message_text := 'text with shadow';

      fg_setcolor(0);
      fg_move(midx+1,midy+1);
      fgf_justify(0,0);
      fgf_print(message_text,16);
      fg_setcolor(19);
      fg_move(midx,midy);
      fgf_print(message_text,16);

      Exit;
    end;

    WM_PAINT:
    begin
      BeginPaint(window,ps);
      fg_vbscale(0,fg_getmaxx,0,fg_getmaxy,0,cxClient-1,0,cyClient-1);
      EndPaint(window,ps);
      Exit;
    end;

    WM_SETFOCUS:
    begin
      fg_realize(hpal);
      InvalidateRect(window,nil,TRUE);
      Exit;
    end;

    WM_SIZE:
    begin
      cxClient := LOWORD(lParam);
      cyClient := HIWORD(lParam);
      Exit;
    end;

    WM_DESTROY:
    begin
      fgf_unload(-1);
      fg_vbclose;
      fg_vbfree(hvb);
      fg_vbfin;
      DeleteObject(hpal);
      ReleaseDC(window,dc);
      PostQuitMessage(0);
      Exit;
    end;

  end;
  WindowProc := DefWindowProc(window,message,wParam,lParam);
end;

{****************************************************************************}

procedure WinMain;

var
  window      : hWnd;
  message     : tMsg;
  WindowClass : tWndClass;

begin
  if hPrevInst = 0 then
  begin
    WindowClass.style         := CS_HREDRAW OR CS_VREDRAW OR CS_OWNDC;
    WindowClass.lpfnWndProc   := @WindowProc;
    WindowClass.cbClsExtra    := 0;
    WindowClass.cbWndExtra    := 0;
    WindowClass.hInstance     := hInstance;
    WindowClass.hIcon         := LoadIcon(0,IDI_APPLICATION);
    WindowClass.hCursor       := LoadCursor(0,IDC_ARROW);
    WindowClass.hbrBackground := 0;
    WindowClass.lpszMenuName  := nil;
    WindowClass.lpszClassName := AppName;
    if not RegisterClass(WindowClass) then Halt(255);
  end;

  window := CreateWindow(
    AppName,                  { window class name }
    'Example FGFW7',          { window caption }
    WS_OVERLAPPEDWINDOW,      { window style }
    CW_USEDEFAULT,            { initial x position }
    CW_USEDEFAULT,            { initial y position }
    CW_USEDEFAULT,            { initial x size }
    CW_USEDEFAULT,            { initial y size }
    0,                        { parent window handle }
    0,                        { window menu handle }
    hInstance,                { program instance handle }
    nil);                     { creation parameters }

  ShowWindow(window,CmdShow);
  UpdateWindow(window);

  while GetMessage(message,0,0,0) do
  begin
    TranslateMessage(message);
    DispatchMessage(message);
  end;

  Halt(message.wParam);
end;

begin
  WinMain;
end.
